/**
 * Provides component for rendering elements written in SJDON notation.
 */
export class SuiWeb
{
    /**
     * Renders the specified {@link data `data`} and appends it to the specified {@link element `element`}.
     *
     * @param {NodeDescriptor} data
     * The node to render written in SJDON notation.
     *
     * @param {HTMLElement} element
     * The element to add the rendered node to.
     *
     * @returns {Node}
     * The resulting element.
     */
    static render(data, element)
    {
        if (Array.isArray(data))
        {
            let descriptor = data[0];
            let args = data.slice(1);

            if (typeof descriptor === "function" || typeof descriptor === "string")
            {
                /**
                 * @type {HTMLElement}
                 */
                let result;

                if (typeof descriptor === "function")
                {
                    /**
                     * @type {any[]}
                     */
                    let arg = [];

                    if (typeof args[0] === "object" && !Array.isArray(args[0]))
                    {
                        arg = [args[0]];
                        args = args.slice(1);
                    }

                    result = /** @type {HTMLElement} */ (SuiWeb.render(descriptor(...arg), element));
                }
                else
                {
                    result = element.ownerDocument.createElement(descriptor);
                    element.appendChild(result);
                }

                for (let arg of args)
                {
                    if (typeof arg === "object" && !Array.isArray(arg))
                    {
                        Object.assign(result, arg);
                    }
                    else
                    {
                        SuiWeb.render(arg, result);
                    }
                }

                return result;
            }
            else
            {
                throw new SyntaxError();
            }
        }
        else if (typeof data === "string")
        {
            return element.appendChild(element.ownerDocument.createTextNode(data));
        }
        else
        {
            throw new SyntaxError();
        }
    }
}
