/**
 * Represents the state of a field.
 */
type CellOwner = "" | "r" | "b";

/**
 * Represents a row of the game field.
 */
type Row = CellOwner[];

/**
 * Represents a game board.
 */
type Board = Row[];

/**
 * Represents the state of a game.
 */
interface IState
{
    /**
     * The number of rounds that have been played.
     */
    turnCount: number;

    /**
     * The board of the game.
     */
    board: Board;
}

/**
 * Represents a node in the SJDON notation.
 */
type NodeDescriptor =
    TextDescriptor |
    ElementDescriptor |
    FunctionNode;

/**
 * Represents a text-node in the SJDON notation.
 */
type TextDescriptor = string;

/**
 * Represents an html-element in the SJDON notation.
 */
type ElementDescriptor = [
    tag: string,
    // eslint-disable-next-line @typescript-eslint/array-type
    ...args: (NodeDescriptor | (Partial<HTMLElement> & Record<string, any>))[]
];

/**
 * Represents a component in the SJDON notation.
 */
type FunctionNode = [
    fn: (...args: any[]) => NodeDescriptor, ...args: any[]
];
